"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoDBKeySpacePipelineStack = void 0;
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_codebuild_1 = require("aws-cdk-lib/aws-codebuild");
const aws_iam_1 = require("aws-cdk-lib/aws-iam");
const pipelines_1 = require("aws-cdk-lib/pipelines");
const App_1 = require("./App");
const Stage_1 = require("./Stage");
class DynamoDBKeySpacePipelineStack extends aws_cdk_lib_1.Stack {
    constructor(scope, id, props) {
        super(scope, id, props);
        const source = pipelines_1.CodePipelineSource.gitHub(`visionsofparadise/${App_1.serviceName}`, 'main', {
            authentication: aws_cdk_lib_1.SecretValue.secretsManager('GITHUB_TOKEN')
        });
        const pipeline = new pipelines_1.CodePipeline(this, 'pipeline', {
            synth: new pipelines_1.ShellStep('synth', {
                input: source,
                installCommands: ['npm i'],
                commands: ['npx cdk synth']
            })
        });
        const testApp = new Stage_1.DynamoDBKeySpaceStage(this, 'testStage', {
            stage: 'test',
            env: {
                region: process.env.CDK_DEFAULT_REGION,
                account: process.env.CDK_DEFAULT_ACCOUNT
            }
        });
        const testStage = pipeline.addStage(testApp);
        const unitTestStep = new pipelines_1.ShellStep('unitTest', {
            input: source,
            installCommands: ['npm ci'],
            commands: ['npm run check', 'npm run test']
        });
        testStage.addPre(unitTestStep);
        const testingPolicyStatement = new aws_iam_1.PolicyStatement({
            effect: aws_iam_1.Effect.ALLOW,
            resources: ['*'],
            actions: [
                'dynamodb:GetItem',
                'dynamodb:Query',
                'dynamodb:Scan',
                'dynamodb:PutItem',
                'dynamodb:UpdateItem',
                'dynamodb:DeleteItem',
                'dynamodb:BatchGetItem',
                'dynamodb:BatchWriteItem'
            ]
        });
        const integrationTestStep = new pipelines_1.CodeBuildStep('integrationTest', {
            commands: ['npm ci', 'export INTEGRATION_TEST=true', 'npm run test'],
            rolePolicyStatements: [testingPolicyStatement],
            envFromCfnOutputs: {
                DYNAMODB_TABLE: testApp.tableName,
                DYNAMODB_SCAN_TABLE: testApp.scanTableName,
                DYNAMODB_RESET_TABLE: testApp.resetTableName
            }
        });
        const publishStep = new pipelines_1.CodeBuildStep('publish', {
            input: source,
            commands: [
                'npm ci',
                'npm run build',
                'npm config set //registry.npmjs.org/:_authToken ${NPM_TOKEN}',
                'npm run patch'
            ],
            buildEnvironment: {
                environmentVariables: {
                    NPM_TOKEN: {
                        type: aws_codebuild_1.BuildEnvironmentVariableType.SECRETS_MANAGER,
                        value: 'NPM_TOKEN'
                    }
                }
            }
        });
        publishStep.addStepDependency(integrationTestStep);
        testStage.addPost(integrationTestStep, publishStep);
    }
}
exports.DynamoDBKeySpacePipelineStack = DynamoDBKeySpacePipelineStack;
