import { BatchGetCommand } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { lowerCaseKeys, upperCaseKeys } from '../util/keyCapitalize';
import { applyDefaults } from '../util/defaults';
import { executeMiddlewares, executeMiddleware } from '../Middleware';
const BATCH_GET_COMMAND_INPUT_DATA_TYPE = 'BatchGetCommandInput';
const BATCH_GET_COMMAND_INPUT_HOOK = ['CommandInput', 'ReadCommandInput', BATCH_GET_COMMAND_INPUT_DATA_TYPE];
const BATCH_GET_COMMAND_OUTPUT_DATA_TYPE = 'BatchGetCommandOutput';
const BATCH_GET_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'ReadCommandOutput',
    BATCH_GET_COMMAND_OUTPUT_DATA_TYPE
];
export class DkBatchGetCommand extends DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: BATCH_GET_COMMAND_INPUT_DATA_TYPE, hooks: BATCH_GET_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: BATCH_GET_COMMAND_OUTPUT_DATA_TYPE, hooks: BATCH_GET_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = applyDefaults(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await executeMiddlewares([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                requestItems: Object.fromEntries(Object.entries(requests).map(([tableName, keysAndAttributes]) => [tableName, upperCaseKeys(keysAndAttributes)])),
                ...rest
            };
            const upperCaseInput = upperCaseKeys(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = lowerCaseKeys(output);
            const { responses, unprocessedKeys, ...rest } = lowerCaseOutput;
            const items = Object.fromEntries(Object.entries(responses || {}).map(([tableName, tableItems]) => {
                const typedTableItems = (tableItems || []);
                return [tableName, typedTableItems];
            }));
            const formattedUnprocessedRequests = Object.fromEntries(Object.entries(unprocessedKeys || {}).map(([tableName, request]) => {
                if (!request)
                    return [tableName, undefined];
                const { Keys, ...unprocessedRest } = request;
                const typedKeys = (Keys || []);
                return [tableName, lowerCaseKeys({ Keys: typedKeys, ...unprocessedRest })];
            }));
            const formattedOutput = {
                ...rest,
                items,
                unprocessedRequests: formattedUnprocessedRequests
            };
            const { data: postMiddlewareOutput } = await executeMiddlewares([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await executeMiddleware('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new BatchGetCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
