import { TransactWriteCommand } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { lowerCaseKeys, upperCaseKeys } from '../util/keyCapitalize';
import { applyDefaults } from '../util/defaults';
import { executeMiddlewares, executeMiddleware } from '../Middleware';
const TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE = 'TransactWriteCommandInput';
const TRANSACT_WRITE_COMMAND_INPUT_HOOK = [
    'CommandInput',
    'WriteCommandInput',
    TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE
];
const TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE = 'TransactWriteCommandOutput';
const TRANSACT_WRITE_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'WriteCommandOutput',
    TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE
];
export class DkTransactWriteCommand extends DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = {
            dataType: TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE,
            hooks: TRANSACT_WRITE_COMMAND_INPUT_HOOK
        };
        this.outputMiddlewareConfig = {
            dataType: TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE,
            hooks: TRANSACT_WRITE_COMMAND_OUTPUT_HOOK
        };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = applyDefaults(this.input, defaults, [
                'returnConsumedCapacity',
                'returnItemCollectionMetrics'
            ]);
            const { data: postMiddlewareInput } = await executeMiddlewares([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                transactItems: requests.map(request => {
                    if (request.type === 'conditionCheck') {
                        return {
                            ConditionCheck: upperCaseKeys(request)
                        };
                    }
                    if (request.type === 'put') {
                        return {
                            Put: upperCaseKeys(request)
                        };
                    }
                    if (request.type === 'delete') {
                        return {
                            Delete: upperCaseKeys(request)
                        };
                    }
                    if (request.type === 'update') {
                        return {
                            Update: upperCaseKeys(request)
                        };
                    }
                    return {};
                }),
                ...rest
            };
            const upperCaseInput = upperCaseKeys(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = lowerCaseKeys(output);
            const { data: postMiddlewareOutput } = await executeMiddlewares([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: lowerCaseOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await executeMiddleware('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            if (postMiddlewareOutput.itemCollectionMetrics) {
                await executeMiddleware('ItemCollectionMetrics', { dataType: 'ItemCollectionMetrics', data: postMiddlewareOutput.itemCollectionMetrics }, middleware);
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new TransactWriteCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
